using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Xml.Linq;

namespace ksp_osm
{
    public struct Node
    {
        public const int IntRescaleConstant = 10_000_000;
        public Node(long id,
                    double lat,
                    double lon,
                    Dictionary<string, string> tags)
        {
            Id = id;
            // 32-bitové floaty nejsou dost dobré a 64-bitové floaty zabírají dvakrát tolik místa a hlavně je bohužel nepodporují geometrické utility ze System.Drawing
            // Můžeme udělat drobnou optimalizaci popsanou na OSM wiki https://wiki.openstreetmap.org/wiki/Node
            // převedeme sousřadnice na celé číslo v rozsahu −1800_000_000 .. 1800_000_000, respektive -900_000_000 .. 900_000_000
            Lat = (int)(lat * IntRescaleConstant);
            Lon = (int)(lon * IntRescaleConstant);
            Tags = tags;
        }

        public long Id { get; }
        public int Lat { get; }
        public int Lon { get; }
        public Dictionary<string, string> Tags { get; }
        public Point Location => new Point(Lon, Lat);

        public static Dictionary<string, string> EmptyTags = new Dictionary<string, string>();

        public static (string key, string value) ReadTag(XElement xml) =>
            ((string)xml.Attribute("k"), (string)xml.Attribute("v"));

        public static Dictionary<string, string> ReadTags(IEnumerable<XElement> elements)
        {
            var tags = elements.Select(ReadTag);
            var dict = tags.ToDictionary(t => string.Intern(t.key), t => string.Intern(t.value));
            //                             ^ toto je drobný trik, který odstraní z paměti duplicitní stringy. Když už je .NET reprezentruje retardovaně jako UTF-16, tak si alespoň na něčem nahrabem :)
            if (dict.Count == 0)
                return EmptyTags;

            return dict;
        }

        private static HashSet<string> InterestingTags = null; // new HashSet<string> { };
        public static Node FromXml(XElement xml)
        {
            return new Node(
                id: (long)xml.Attribute("id"),
                lat: (double)xml.Attribute("lat"),
                lon: (double)xml.Attribute("lon"),
                tags: xml.HasElements ? ReadTags(xml.Elements("tag")) : EmptyTags
            );
        }

        public override string ToString()
        {
            return $"Node {Id} at [{this.Lat}, {this.Lon}]";
        }
    }
}
