using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Xml;
using System.Xml.Linq;

namespace ksp_osm
{
    public static class OsmReader
    {
        public static TextReader OpenFile(string path)
        {
            Stream s = File.OpenRead(path);

            if (int.TryParse(Path.GetExtension(path).TrimStart('.'), out var _))
                path = Path.GetFileNameWithoutExtension(path);

            if (path.EndsWith(".gz"))
                s = new GZipStream(s, CompressionMode.Decompress);
            return new StreamReader(s);
        }

        public static IEnumerable<XElement> LoadElements(TextReader text, Func<string, Lazy<long>, bool> filter = null)
        {
            // XmlReader je streamové parsovadlo XML v .NETu
            using (var reader = XmlReader.Create(text, new XmlReaderSettings { IgnoreComments = true, IgnoreWhitespace = true }))
            {
                reader.MoveToContent();
                reader.Read();
                while (!reader.EOF)
                {
                    if (reader.NodeType == XmlNodeType.Element)
                    {
                        // před načtením celého elementu si můžu přečíst jenom název elementu a atributy a zkontrolovat si, že ho fakt chci načítat celý
                        if (filter == null || filter(reader.Name, new Lazy<long>(() => long.Parse(reader.GetAttribute("id")))))
                        {
                            // načteme element, protože nás zajímá
                            var el = (XElement)XNode.ReadFrom(reader);
                            // a pak ho vrátíme jako částečný výsledek
                            // tato metoda je trochu speciální, protože vrací libovolné množství XElementů
                            // a spočítá je až když je to potřeba, takže nám nebudou zabírat paměť všechny najednou
                            yield return el;
                        }
                        else
                        {
                            // když nás nezajímá, přeskočíme
                            reader.Skip();
                        }
                    }
                    else
                    {
                        // když to není element, tak ho přeskočíme
                        reader.Read();
                    }
                }
            }
        }

        public static IEnumerable<XElement> LoadElements(string file, Func<string, Lazy<long>, bool> filter = null)
        {
            return LoadElements(OpenFile(file), filter);
        }

    }
}
